require 'fastlane/action'
require_relative '../helper/discord_webhook_notifier_helper'

module Fastlane
  module Actions
    class DiscordWebhookNotifierAction < Action
      def self.run(params)

        # Config paramters
        webhook_url = params[:webhook_url]
        message = params[:message]
        parameters = params[:parameters]
        if (defined?(params[:build_url]) && params[:build_url].instance_of?(String))
          build_url = params[:build_url].to_s
        end


        parametersString = ""
        parameters.each do |key, value|
          parametersString += " \\n #{key}:"
          parametersString +=" #{value}"
        end

        # Create cURL command
        curl_command = [
          'curl -X POST --data \'{"content": "' + message,
          '```',
          parametersString,
          '```',
          params[:build_url].instance_of?(String) ? 'Build URL: <' + build_url + '>' : '', ## The <> are used to disable auto embed
          '"}\' --header "Content-Type:application/json" ' + webhook_url
        ].join

        # Send notification
        sh curl_command
        UI.success("Notification sent!")
      end

      def self.description
        "Notify a Discord channel easily when a build breake"
      end

      def self.authors
        ["Samuel B Bispo"]
      end

      def self.return_value
        # If your method provides a return value, you can describe here what it does
      end

      def self.details
        # Optional:
        "Using this plugin you can quickly send notification through webhooks to an Discord channel. It was created to notify a channel when a build breakes."
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :webhook_url,
                                  env_name: "DISCORD_WEBHOOK_NOTIFIER_WEBHOOK_URL",
                               description: "The destination webook URL to where the api call will be send. https://discordjs.guide/popular-topics/webhooks.html#creating-webhooks",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :message,
                                  env_name: "DISCORD_WEBHOOK_NOTIFIER_MESSAGE",
                               description: "A message to be shown as an header to the notification",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :parameters,
                                  env_name: "DISCORD_WEBHOOK_NOTIFIER_PARAMETERS",
                               description: "A dictonary of key/value to help give context about the message",
                                  optional: false,
                                      type: Hash),
          FastlaneCore::ConfigItem.new(key: :build_url,
                                  env_name: "DISCORD_WEBHOOK_NOTIFIER_BUILD_URL",
                               description: "A link to where the build was done",
                                  optional: true,
                                      type: String)
        ]
      end

      def self.is_supported?(platform)
        # Adjust this if your plugin only works for a particular platform (iOS vs. Android, for example)
        # See: https://docs.fastlane.tools/advanced/#control-configuration-by-lane-and-by-platform
        #
        # [:ios, :mac, :android].include?(platform)
        true
      end
    end
  end
end
